//==============================================================================
// Product name: DBMemBenchmark
// Copyright 2003 AidAim Software LLC.
//==============================================================================
unit MainUnit;

{DEFINE NO_PROGRESS}

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, DB, Spin, ACRMain, Grids,
  FormProgress, EasyTable;

type
  TForm1 = class(TForm)
    bnTestACRTable: TButton;
    Label4: TLabel;
    seRecordCount: TSpinEdit;
    bnTestAll: TButton;
    Button5: TButton;
    GroupBox1: TGroupBox;
    sgResults: TStringGrid;
    ACRTable1: TACRTable;
    ACRDatabase1: TACRDatabase;
    EasyDatabase1: TEasyDatabase;
    EasyTable1: TEasyTable;
    bnEasyTable: TButton;
    procedure Button5Click(Sender: TObject);
    procedure bnTestACRTableClick(Sender: TObject);
    procedure bnTestAllClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure bnEasyTableClick(Sender: TObject);
  private
    FTime:  Cardinal;
    FTime1: Cardinal;
  private
    { Private declarations }
    procedure ShowProgress(
                            Dataset:    TDataset;
                            Msg:        String;
                            TestNo:     Integer;
                            UseIndexes: Boolean
                         );
    procedure WriteToLog(
                            Dataset:    TDataset;
                            Msg:        String;
                            TestNo:     Integer;
                            Time:       Cardinal;
                            UseIndexes: Boolean
                         );
    procedure WriteToFile(Str: String);
    procedure CreateTables(UseIndexes: Boolean);
    procedure InsertTest(Dataset: TDataset; UseIndexes: Boolean);
    procedure ReadTest(Dataset: TDataset; UseIndexes: Boolean);
    procedure EditTest(Dataset: TDataset; UseIndexes: Boolean);
    procedure DeleteTest(Dataset: TDataset; UseIndexes: Boolean);
    procedure RecNoTest(Dataset: TDataset; UseIndexes: Boolean);
    procedure RecNoFilteredTest(Dataset: TDataset; UseIndexes: Boolean);
    procedure LocateByIDTest(Dataset: TDataset; UseIndexes: Boolean);
    procedure LocateByFIntegerTest(Dataset: TDataset; UseIndexes: Boolean);
    procedure LocateByFStringTest(Dataset: TDataset; UseIndexes: Boolean);

    procedure TestAll(Dataset: TDataset; UseIndexes: Boolean);
  public
    { Public declarations }
  end;

const
  tests = 9;
var
  Form1: TForm1;
  F: text;

const NO_TIME = $FFFFFFFF;

implementation

{$R *.dfm}

function GenerateString(
                       Len : Integer // serial length
                       ) : String; // returns serial
var i,x : integer;
    s : string;
    c : char;
begin
 s := '';

 for i := 1 to len do
  begin
   x := Random(101);
   if ((x mod 2) =  0) then
    c := chr(65+(Random(260000000) mod 26))
   else
    c := chr(48+(Random(100000000) mod 10));

   s := s + c;
  end; //len
 result := s;
end; // GenerateString


procedure TForm1.ShowProgress(
                            Dataset:    TDataset;
                            Msg:        String;
                            TestNo:     Integer;
                            UseIndexes: Boolean
                         );
var EngineName: String;
    f:          Double;
begin
{$IFDEF NO_PROGRESS}
Exit;
{$ENDIF}
   if (UseIndexes) then
    Msg := Msg + ' with indexes'
   else
    Msg := Msg + ' without indexes';
 if (Dataset = ACRTable1) then
  EngineName := sgResults.Cells[1,0];
 if (Dataset = EasyTable1) then
  EngineName := sgResults.Cells[2,0];
 if (UseIndexes) then
  TestNo := TestNo+6;
 f := TestNo-1;
 fmProgress.Init(Msg,EngineName,Round(f / 14.0 * 100.0));
end; // ShowProgress


procedure TForm1.WriteToLog(
                            Dataset:    TDataset;
                            Msg:        String;
                            TestNo:     Integer;
                            Time:       Cardinal;
                            UseIndexes: Boolean
                            );
var Column: Integer;
begin
   if (UseIndexes) then
    Msg := Msg + ' with indexes'
   else
    Msg := Msg + ' without indexes';
 if (Dataset = ACRTable1) then
  Column := 1;
 if (Dataset = EasyTable1) then
  Column := 2;
 if (UseIndexes) then
  Inc(TestNo,tests);
 if (Time = NO_TIME) then
  begin
   sgResults.Cells[0,TestNo] := Msg + ' ...';
   sgResults.Cells[Column,TestNo] := '';
  end
 else
  begin
   sgResults.Cells[0,TestNo] := Msg;
   sgResults.Cells[Column,TestNo] := IntToStr(Time) + ' ms';
   if TestNo=1 then WriteToFile(#13+#10);
   WriteToFile(IntToStr(Time));
  end;
 Application.ProcessMessages;
end;

procedure TForm1.CreateTables(UseIndexes: Boolean);
begin
 ACRDatabase1.Close;
 if (ACRDatabase1.Exists) then
   ACRDatabase1.DeleteDatabase;
 ACRDatabase1.CreateDatabase;
 ACRDatabase1.Open;
 ACRTable1.Close;
 ACRTable1.FieldDefs.Clear;
 ACRTable1.FieldDefs.Add('ID',ftAutoInc);
 ACRTable1.FieldDefs.Add('FString',ftString,100);
 ACRTable1.FieldDefs.Add('FInteger',ftInteger,0);
 ACRTable1.IndexDefs.Clear;
 if (UseIndexes) then
  begin
   ACRTable1.IndexDefs.Add('index1','ID',[]);
   ACRTable1.IndexDefs.Add('index2','FString',[]);
   ACRTable1.IndexDefs.Add('index3','FInteger',[]);
  end;
 ACRTable1.CreateTable;
 ACRTable1.Open;

 EasyDatabase1.Close;
 if (EasyDatabase1.Exists) then
   EasyDatabase1.DeleteDatabase;
 EasyDatabase1.CreateDatabase;
 EasyDatabase1.Open;
 EasyTable1.Close;
 EasyTable1.FieldDefs.Clear;
 EasyTable1.FieldDefs.Add('ID',ftAutoInc);
 EasyTable1.FieldDefs.Add('FString',ftString,100);
 EasyTable1.FieldDefs.Add('FInteger',ftInteger,0);
 EasyTable1.IndexDefs.Clear;
 if (UseIndexes) then
  begin
   EasyTable1.IndexDefs.Add('index1','ID',[]);
   EasyTable1.IndexDefs.Add('index2','FString',[]);
   EasyTable1.IndexDefs.Add('index3','FInteger',[]);
  end;
 EasyTable1.CreateTable;
 EasyTable1.Open;
end; //


procedure TForm1.InsertTest(Dataset: TDataset; UseIndexes: Boolean);
var i,x,y: Integer;
    str:   String;
begin
 WriteToLog(Dataset,'Insert',1,NO_TIME,UseIndexes);
 ShowProgress(Dataset,'Insert',1,UseIndexes);
 y := seRecordCount.Value div 100;
 FTime := GetTickCount;
 for i := 1 to seRecordCount.Value do
  begin
   FTime1 := GetTickCount;
   x := random (MAXInt) mod (seRecordCount.Value div 10);
   str := GenerateString(100);
   FTime := FTime + (GetTickCount - FTime1);
   Dataset.Insert;
   Dataset.FieldByName('FInteger').AsInteger := x;
   Dataset.FieldByName('FString').AsString := str;
   Dataset.Post;
{$IFNDEF NO_PROGRESS}
   if ((i mod y) = 0) then
    begin
     fmProgress.gCurrent.Progress:= Round((i / seRecordCount.Value )* 100.0);
     Application.ProcessMessages;
     if (FormProgress.CloseFlag) then Exit;
    end;
{$ENDIF}
  end;
 FTime := GetTickCount - FTime;
 WriteToLog(Dataset,'Insert',1,FTime,UseIndexes);
end; //


procedure TForm1.ReadTest(Dataset: TDataset; UseIndexes: Boolean);
var i,y,x:  Integer;
    str:    String;
begin
 WriteToLog(Dataset,'Read',2,NO_TIME,UseIndexes);
 ShowProgress(Dataset,'Read',2,UseIndexes);
 y := seRecordCount.Value div 100;
 Dataset.First;
 FTime := GetTickCount;
 for i := 1 to Dataset.RecordCount do
  begin
   x := Dataset.FieldByName('FInteger').AsInteger;
   str := Dataset.FieldByName('FString').AsString;
   Dataset.Next;
{$IFNDEF NO_PROGRESS}
   if ((i mod y) = 0) then
    begin
     fmProgress.gCurrent.Progress:= Round((i / seRecordCount.Value )* 100.0);
     Application.ProcessMessages;
     if (FormProgress.CloseFlag) then Exit;
    end;
{$ENDIF}
  end;
 FTime := GetTickCount - FTime;
 WriteToLog(Dataset,'Read',2,FTime,UseIndexes);
end; //


procedure TForm1.EditTest(Dataset: TDataset; UseIndexes: Boolean);
var i,y,x:  Integer;
    str:    String;
begin
 WriteToLog(Dataset,'Edit',3,NO_TIME,UseIndexes);
 ShowProgress(Dataset,'Edit',3,UseIndexes);
 y := seRecordCount.Value div 100;
 FTime := GetTickCount;
 for i := 1 to seRecordCount.Value do
  begin
   Dataset.RecNo := i;
   FTime1 := GetTickCount;
   x := random (MAXInt) mod (seRecordCount.Value div 10);
   str := GenerateString(100);
   FTime := FTime + (GetTickCount - FTime1);
   Dataset.Edit;
   Dataset.FieldByName('FInteger').AsInteger := x;
   Dataset.FieldByName('FString').AsString := str;
   Dataset.Post;
{$IFNDEF NO_PROGRESS}
   if ((i mod y) = 0) then
    begin
     fmProgress.gCurrent.Progress:= Round((i / seRecordCount.Value )* 100.0);
     Application.ProcessMessages;
     if (FormProgress.CloseFlag) then Exit;
    end;
{$ENDIF}
  end;
 FTime := GetTickCount - FTime;
 WriteToLog(Dataset,'Edit',3,FTime,UseIndexes);
end; //


procedure TForm1.DeleteTest(Dataset: TDataset; UseIndexes: Boolean);
var RecCount: Double;
    i,y:      Integer;
begin
 WriteToLog(Dataset,'Delete',9,NO_TIME,UseIndexes);
 ShowProgress(Dataset,'Delete',9,UseIndexes);
 RecCount := dataset.RecordCount;
 i := 1;
 y := dataset.RecordCount div 100;
 FTime := GetTickCount;
 while (dataset.RecordCount > 0) do
  begin
   dataset.Delete;
   Inc(i);
{$IFNDEF NO_PROGRESS}
   if ((i mod y) = 0) then
    begin
     fmProgress.gCurrent.Progress:= Round(((RecCount - dataset.recordCount) / RecCount)* 100.0);
     Application.ProcessMessages;
     if (FormProgress.CloseFlag) then Exit;
    end;
{$ENDIF}
  end;
 FTime := GetTickCount - FTime;
 WriteToLog(Dataset,'Delete',9,FTime,UseIndexes);
end; //


procedure TForm1.LocateByIDTest(Dataset: TDataset; UseIndexes: Boolean);
var i,x,y,OldRecNo: Integer;
begin
 WriteToLog(Dataset,'Locate By ID',6,NO_TIME,UseIndexes);
 ShowProgress(Dataset,'Locate By ID',6,UseIndexes);
 y := 10;
 FTime := GetTickCount;
 for i := 1 to 5000 do
  begin
   Dataset.RecNo := i;
   OldRecNo := Dataset.RecNo;
   x := Dataset.FieldByName('ID').AsInteger;
   Dataset.RecNo := 1;
   if (not Dataset.Locate('ID',x,[])) then
     raise Exception.Create('Locate By ID test failed');
   if (Dataset.FieldByName('ID').AsInteger <> x) then
     raise Exception.Create('Locate By ID test failed invalid record found');
{$IFNDEF NO_PROGRESS}
   if ((i mod y) = 0) then
    begin
     fmProgress.gCurrent.Progress:= Round((i / 1000 )* 100.0);
     Application.ProcessMessages;
     if (FormProgress.CloseFlag) then Exit;
    end;
{$ENDIF}
  end;
 FTime := GetTickCount - FTime;
 WriteToLog(Dataset,'Locate By ID',6,FTime,UseIndexes);
end; //


procedure TForm1.RecNoTest(Dataset: TDataset; UseIndexes: Boolean);
var i,x,y,low,medium,big: Integer;
begin
 WriteToLog(Dataset,'Navigate by RecNo',4,NO_TIME,UseIndexes);
 ShowProgress(Dataset,'Navigate by RecNo',4,UseIndexes);
 y := 10;
 low := dataset.RecordCount div 4;
 medium := dataset.RecordCount div 2;
 big := dataset.RecordCount - low;
 FTime := GetTickCount;
 for i := 1 to 3000 do
  begin
   Dataset.RecNo := low;
   Dataset.RecNo := medium;
   Dataset.RecNo := big;
   Dataset.RecNo := medium;
{$IFNDEF NO_PROGRESS}
   if ((i mod y) = 0) then
    begin
     fmProgress.gCurrent.Progress:= Round((i / 1000 )* 100.0);
     Application.ProcessMessages;
     if (FormProgress.CloseFlag) then Exit;
    end;
{$ENDIF}
  end;
 FTime := GetTickCount - FTime;
 WriteToLog(Dataset,'Navigate by RecNo',4,FTime,UseIndexes);
end; //


procedure TForm1.RecNoFilteredTest(Dataset: TDataset; UseIndexes: Boolean);
var i,x,y,low,medium,big: Integer;
begin
 WriteToLog(Dataset,'Navigate by RecNo with filter',5,NO_TIME,UseIndexes);
 ShowProgress(Dataset,'Navigate by RecNo with filter',5,UseIndexes);
 y := 10;
 low := dataset.RecordCount div 4;
 medium := dataset.RecordCount div 2;
 big := dataset.RecordCount - low;
 Dataset.Filter := 'ID>=0';
 Dataset.Filtered := True;
 FTime := GetTickCount;
 for i := 1 to 3000 do
  begin
   Dataset.RecNo := low;
   Dataset.RecNo := medium;
   Dataset.RecNo := big;
   Dataset.RecNo := medium;
{$IFNDEF NO_PROGRESS}
   if ((i mod y) = 0) then
    begin
     fmProgress.gCurrent.Progress:= Round((i / 1000 )* 100.0);
     Application.ProcessMessages;
     if (FormProgress.CloseFlag) then Exit;
    end;
{$ENDIF}
  end;
 FTime := GetTickCount - FTime;
 Dataset.Filtered := False;
 WriteToLog(Dataset,'Navigate by RecNo with filter',5,FTime,UseIndexes);
end; //


procedure TForm1.LocateByFIntegerTest(Dataset: TDataset; UseIndexes: Boolean);
var i,x,y,OldRecNo: Integer;
begin
 WriteToLog(Dataset,'Locate By FInteger',7,NO_TIME,UseIndexes);
 ShowProgress(Dataset,'Locate By FInteger',7,UseIndexes);
 y := 10;
 FTime := GetTickCount;
 for i := 1 to 1000 do
  begin
   Dataset.RecNo := i;
   OldRecNo := Dataset.RecNo;
   x := Dataset.FieldByName('FInteger').AsInteger;
   Dataset.RecNo := 1;
   if (not Dataset.Locate('FInteger',x,[])) then
     raise Exception.Create('Locate By FInteger test failed');
   if (Dataset.FieldByName('FInteger').AsInteger <> x) then
     raise Exception.Create('Locate By FInteger test failed record found');
{$IFNDEF NO_PROGRESS}
   if ((i mod y) = 0) then
    begin
     fmProgress.gCurrent.Progress:= Round((i / 1000 )* 100.0);
     Application.ProcessMessages;
     if (FormProgress.CloseFlag) then Exit;
    end;
{$ENDIF}
  end;
 FTime := GetTickCount - FTime;
 WriteToLog(Dataset,'Locate By FInteger',7,FTime,UseIndexes);
end; //


procedure TForm1.LocateByFStringTest(Dataset: TDataset; UseIndexes: Boolean);
var i,x,y,OldRecNo: Integer;
    s: string;
begin
 WriteToLog(Dataset,'Locate By FString',8,NO_TIME,UseIndexes);
 ShowProgress(Dataset,'Locate By FString',8,UseIndexes);
 y := 10;
 FTime := GetTickCount;
 for i := 1 to 1000 do
  begin
   Dataset.RecNo := i;
   OldRecNo := Dataset.RecNo;
   s := Dataset.FieldByName('FString').AsString;
   Dataset.RecNo := 1;
   if (not Dataset.Locate('FString',s,[])) then
     raise Exception.Create('Locate By FString test failed');
   if (Dataset.FieldByName('FString').AsString <> s) then
     raise Exception.Create('Locate By FString test failed record found');
{$IFNDEF NO_PROGRESS}
   if ((i mod y) = 0) then
    begin
     fmProgress.gCurrent.Progress:= Round((i / 1000 )* 100.0);
     Application.ProcessMessages;
     if (FormProgress.CloseFlag) then Exit;
    end;
{$ENDIF}
  end;
 FTime := GetTickCount - FTime;
 WriteToLog(Dataset,'Locate By FString',8,FTime,UseIndexes);
end; //


procedure TForm1.TestAll(Dataset: TDataset; UseIndexes: Boolean);
begin
 InsertTest(Dataset,UseIndexes);
 if (FormProgress.CloseFlag) then Exit;
 ReadTest(Dataset,UseIndexes);
 if (FormProgress.CloseFlag) then Exit;
 EditTest(Dataset,UseIndexes);
 if (FormProgress.CloseFlag) then Exit;
 RecNoTest(Dataset,UseIndexes);
 if (FormProgress.CloseFlag) then Exit;
 RecNoFilteredTest(Dataset,UseIndexes);
 if (FormProgress.CloseFlag) then Exit;
 LocateByIDTest(Dataset,UseIndexes);
 if (FormProgress.CloseFlag) then Exit;
 LocateByFIntegerTest(Dataset,UseIndexes);
 if (FormProgress.CloseFlag) then Exit;
 LocateByFStringTest(Dataset,UseIndexes);
 if (FormProgress.CloseFlag) then Exit;
 DeleteTest(Dataset,UseIndexes);
 if (FormProgress.CloseFlag) then Exit;
end; //


procedure TForm1.Button5Click(Sender: TObject);
begin
 Application.Terminate;
 Application.ProcessMessages;
 Close;
end;

procedure TForm1.bnTestACRTableClick(Sender: TObject);
begin
 CreateTables(False);
 TestAll(ACRTable1,False);
 if (FormProgress.CloseFlag) then
  begin
   fmProgress.Hide;
   Exit;
  end;
 CreateTables(True);
 TestAll(ACRTable1,True);
 fmProgress.gOverall.Progress := 100;
 Application.ProcessMessages;
 fmProgress.Hide;
end;

procedure TForm1.bnTestAllClick(Sender: TObject);
begin
 bnTestACRTableClick(Sender);
 if (FormProgress.CloseFlag) then Exit;
 bnEasyTableClick(Sender);
end;


procedure TForm1.FormCreate(Sender: TObject);
begin
 sgResults.RowCount := tests * 2 + 1;
 sgResults.ColWidths[0] := 245;
 sgResults.Cells[0,0] := 'Test operation';
 sgResults.Cells[1,0] := 'ACRTable';
 sgResults.Cells[2,0] := 'EasyTable';
end;

procedure TForm1.bnEasyTableClick(Sender: TObject);
begin
 CreateTables(False);
 TestAll(EasyTable1,False);
 if (FormProgress.CloseFlag) then
  begin
   fmProgress.Hide;
   Exit;
  end;
 CreateTables(True);
 TestAll(EasyTable1,True);
 fmProgress.gOverall.Progress := 100;
 Application.ProcessMessages;
 fmProgress.Hide;
end;

procedure TForm1.WriteToFile(Str: String);
//procedure TForm1.bnSaveClick(Sender: TObject);
begin
{
 AssignFile(F,'c:\test\test.csv');
 append(F);
 write(F,Str+',');
 CloseFile(F);
}
end;

initialization

 Randomize;

end.
